@@ -22,13 +22,20 @@ import io.netty.buffer.ByteBuf;
 import io.netty.buffer.Unpooled;
 import io.netty.channel.ChannelPromise;
 import org.apache.bookkeeper.mledger.Entry;
+import org.apache.bookkeeper.mledger.Position;
 import org.apache.bookkeeper.mledger.impl.EntryImpl;
 import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
+import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.ServiceConfiguration;
-import org.apache.pulsar.broker.service.*;
 import org.apache.pulsar.common.api.proto.PulsarApi;
 import org.apache.pulsar.common.api.proto.PulsarApi.KeySharedMeta;
+import org.apache.pulsar.broker.service.BrokerService;
+import org.apache.pulsar.broker.service.Consumer;
+import org.apache.pulsar.broker.service.EntryBatchIndexesAcks;
+import org.apache.pulsar.broker.service.EntryBatchSizes;
+import org.apache.pulsar.broker.service.RedeliveryTracker;
+import org.apache.pulsar.broker.service.StickyKeyConsumerSelector;
 import org.apache.pulsar.common.protocol.Commands;
 import org.apache.pulsar.common.protocol.Markers;
 import org.mockito.ArgumentCaptor;
@@ -43,13 +50,32 @@ import org.testng.annotations.Test;
 
 import java.lang.reflect.Field;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.Optional;
+import java.util.Queue;
+import java.util.Set;
+import java.util.concurrent.ConcurrentLinkedQueue;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.stream.Collectors;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.pulsar.common.protocol.Commands.serializeMetadataAndPayload;
 import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.anyBoolean;
+import static org.mockito.Mockito.anyInt;
+import static org.mockito.Mockito.anyList;
+import static org.mockito.Mockito.anyLong;
+import static org.mockito.Mockito.anySet;
+import static org.mockito.Mockito.anyString;
+import static org.mockito.Mockito.argThat;
+import static org.mockito.Mockito.eq;
+import static org.mockito.Mockito.doAnswer;
+import static org.mockito.Mockito.doReturn;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.fail;
 
@@ -279,6 +305,123 @@ public class PersistentStickyKeyDispatcherMultipleConsumersTest {
         );
     }
 
+    @Test(timeOut = 30000)
+    public void testMessageRedelivery() throws Exception {
+        final Queue<Position> actualEntriesToConsumer1 = new ConcurrentLinkedQueue<>();
+        final Queue<Position> actualEntriesToConsumer2 = new ConcurrentLinkedQueue<>();
+
+        final Queue<Position> expectedEntriesToConsumer1 = new ConcurrentLinkedQueue<>();
+        expectedEntriesToConsumer1.add(PositionImpl.get(1, 1));
+        final Queue<Position> expectedEntriesToConsumer2 = new ConcurrentLinkedQueue<>();
+        expectedEntriesToConsumer2.add(PositionImpl.get(1, 2));
+        expectedEntriesToConsumer2.add(PositionImpl.get(1, 3));
+
+        final AtomicInteger remainingEntriesNum = new AtomicInteger(
+                expectedEntriesToConsumer1.size() + expectedEntriesToConsumer2.size());
+
+        // Messages with key1 are routed to consumer1 and messages with key2 are routed to consumer2
+        final List<Entry> allEntries = new ArrayList<>();
+        allEntries.add(EntryImpl.create(1, 1, createMessage("message1", 1, "key2")));
+        allEntries.add(EntryImpl.create(1, 2, createMessage("message2", 2, "key1")));
+        allEntries.add(EntryImpl.create(1, 3, createMessage("message3", 3, "key1")));
+        allEntries.forEach(entry -> ((EntryImpl) entry).retain());
+
+        final List<Entry> redeliverEntries = new ArrayList<>();
+        redeliverEntries.add(allEntries.get(0)); // message1
+        final List<Entry> readEntries = new ArrayList<>();
+        readEntries.add(allEntries.get(2)); // message3
+
+        final Consumer consumer1 = mock(Consumer.class);
+        doReturn("consumer1").when(consumer1).consumerName();
+        // Change availablePermits of consumer1 to 0 and then back to normal
+        when(consumer1.getAvailablePermits()).thenReturn(0).thenReturn(10);
+        doReturn(true).when(consumer1).isWritable();
+        doAnswer(invocationOnMock -> {
+            @SuppressWarnings("unchecked")
+            List<Entry> entries = (List<Entry>) invocationOnMock.getArgument(0);
+            for (Entry entry : entries) {
+                remainingEntriesNum.decrementAndGet();
+                actualEntriesToConsumer1.add(entry.getPosition());
+            }
+            return channelMock;
+        }).when(consumer1).sendMessages(anyList(), any(EntryBatchSizes.class), any(EntryBatchIndexesAcks.class),
+                anyInt(), anyLong(), anyLong(), any(RedeliveryTracker.class));
+
+        final Consumer consumer2 = mock(Consumer.class);
+        doReturn("consumer2").when(consumer2).consumerName();
+        when(consumer2.getAvailablePermits()).thenReturn(10);
+        doReturn(true).when(consumer2).isWritable();
+        doAnswer(invocationOnMock -> {
+            @SuppressWarnings("unchecked")
+            List<Entry> entries = (List<Entry>) invocationOnMock.getArgument(0);
+            for (Entry entry : entries) {
+                remainingEntriesNum.decrementAndGet();
+                actualEntriesToConsumer2.add(entry.getPosition());
+            }
+            return channelMock;
+        }).when(consumer2).sendMessages(anyList(), any(EntryBatchSizes.class), any(EntryBatchIndexesAcks.class),
+                anyInt(), anyLong(), anyLong(), any(RedeliveryTracker.class));
+
+        persistentDispatcher.addConsumer(consumer1);
+        persistentDispatcher.addConsumer(consumer2);
+
+        final Field totalAvailablePermitsField = PersistentDispatcherMultipleConsumers.class
+                .getDeclaredField("totalAvailablePermits");
+        totalAvailablePermitsField.setAccessible(true);
+        totalAvailablePermitsField.set(persistentDispatcher, 1000);
+
+        final Field redeliveryMessagesField = PersistentDispatcherMultipleConsumers.class
+                .getDeclaredField("redeliveryMessages");
+        redeliveryMessagesField.setAccessible(true);
+        MessageRedeliveryController redeliveryMessages = (MessageRedeliveryController) redeliveryMessagesField
+                .get(persistentDispatcher);
+        redeliveryMessages.add(allEntries.get(0).getLedgerId(), allEntries.get(0).getEntryId(),
+                getStickyKeyHash(allEntries.get(0))); // message1
+        redeliveryMessages.add(allEntries.get(1).getLedgerId(), allEntries.get(1).getEntryId(),
+                getStickyKeyHash(allEntries.get(1))); // message2
+
+        // Mock Cursor#asyncReplayEntries
+        doAnswer(invocationOnMock -> {
+            @SuppressWarnings("unchecked")
+            Set<Position> positions = (Set<Position>) invocationOnMock.getArgument(0);
+            List<Entry> entries = allEntries.stream().filter(entry -> positions.contains(entry.getPosition()))
+                    .collect(Collectors.toList());
+            if (!entries.isEmpty()) {
+                ((PersistentStickyKeyDispatcherMultipleConsumers) invocationOnMock.getArgument(1))
+                        .readEntriesComplete(entries, PersistentStickyKeyDispatcherMultipleConsumers.ReadType.Replay);
+            }
+            return Collections.emptySet();
+        }).when(cursorMock).asyncReplayEntries(anySet(), any(PersistentStickyKeyDispatcherMultipleConsumers.class),
+                eq(PersistentStickyKeyDispatcherMultipleConsumers.ReadType.Replay), anyBoolean());
+
+        // Mock Cursor#asyncReadEntriesOrWait
+        doAnswer(invocationOnMock -> {
+            ((PersistentStickyKeyDispatcherMultipleConsumers) invocationOnMock.getArgument(2))
+                    .readEntriesComplete(readEntries, PersistentStickyKeyDispatcherMultipleConsumers.ReadType.Normal);
+            return null;
+        }).when(cursorMock).asyncReadEntriesOrWait(anyInt(), anyLong(),
+                any(PersistentStickyKeyDispatcherMultipleConsumers.class),
+                eq(PersistentStickyKeyDispatcherMultipleConsumers.ReadType.Normal));
+
+        // (1) Run sendMessagesToConsumers
+        // (2) Attempts to send message1 to consumer1 but skipped because availablePermits is 0
+        // (3) Change availablePermits of consumer1 to 10
+        // (4) Run readMoreEntries internally
+        // (5) Run sendMessagesToConsumers internally
+        // (6) Attempts to send message3 to consumer2 but skipped because redeliveryMessages contains message2
+        persistentDispatcher.sendMessagesToConsumers(PersistentStickyKeyDispatcherMultipleConsumers.ReadType.Replay,
+                redeliverEntries);
+        while (remainingEntriesNum.get() > 0) {
+            // (7) Run readMoreEntries and resend message1 to consumer1 and message2-3 to consumer2
+            persistentDispatcher.readMoreEntries();
+        }
+
+        assertEquals(actualEntriesToConsumer1, expectedEntriesToConsumer1);
+        assertEquals(actualEntriesToConsumer2, expectedEntriesToConsumer2);
+
+        allEntries.forEach(entry -> entry.release());
+    }
+
     private ByteBuf createMessage(String message, int sequenceId) {
         return createMessage(message, sequenceId, "testKey");
     }
@@ -292,4 +435,9 @@ public class PersistentStickyKeyDispatcherMultipleConsumersTest {
         messageMetadata.setPublishTime(System.currentTimeMillis());
         return serializeMetadataAndPayload(Commands.ChecksumType.Crc32c, messageMetadata.build(), Unpooled.copiedBuffer(message.getBytes(UTF_8)));
     }
+
+    private int getStickyKeyHash(Entry entry) {
+        byte[] stickyKey = Commands.peekStickyKey(entry.getDataBuffer(), topicName, subscriptionName);
+        return StickyKeyConsumerSelector.makeStickyKeyHash(stickyKey);
+    }
 }
